﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DeckLinkAPI;
using System.IO;
using System.IO.Pipes;
using System.Diagnostics;

namespace ClassLibrary1
{
    class CH264 : DeckLinkAPI.IBMDStreamingDeviceNotificationCallback, DeckLinkAPI.IBMDStreamingH264InputCallback
    {
        CBMDStreamingDiscovery m_streamingDiscovery;   //BM Streaming Discovery Object        
        IDeckLink m_streamingDevice;       //BM IDeckLink Object
        IBMDStreamingDeviceInput m_streamingDeviceInput;   //BM StreamingDevice Input object
        _BMDStreamingDeviceMode m_deviceMode;            //BM DeviceMode 
        _BMDVideoConnection m_inputConnector;        //BM Device Input
        _BMDDisplayMode m_inputMode;         //BM Display mode object 
        List<IBMDStreamingVideoEncodingMode> m_encodingModes;
        IBMDStreamingVideoEncodingMode m_CurrentEncodingMode;

        long m_bitrate = 4000;

        bool m_playing;             //Flag used to denot if we are previewing      
        string m_status;
        string m_DispalyNameWithStatus;
        string m_deviceModelName;
        string m_displayModeName;

        FileStream m_sw;
      
        public bool Init()
        {
            Debug.WriteLine("Init");
            UpdateForNoDevice();

            m_streamingDiscovery = new CBMDStreamingDiscovery();
            m_streamingDiscovery.InstallDeviceNotifications(this);
            return true;
        }


        private void UpdateForNoDevice()
        {
            Debug.WriteLine("UpdateForNoDevice");
            m_deviceMode = _BMDStreamingDeviceMode.bmdStreamingDeviceUnknown;
            m_inputMode = _BMDDisplayMode.bmdModeUnknown;

            m_encodingModes = new List<IBMDStreamingVideoEncodingMode>();
            m_CurrentEncodingMode = null;

        }



        public bool StartRecording(string fileName)
        {
            Debug.WriteLine("StartRecording");
            StopPreview();

            m_sw = new FileStream(fileName, System.IO.FileMode.Create);

            StartPreview();
            return true;
        }

        public bool StopRecording()
        {
            Debug.WriteLine("StopRecording");
            StopPreview();
            StartPreview();
            return true;
        }

        public bool StartPreview()
        {
            Debug.WriteLine("StartPreview");
            if (m_streamingDevice == null)
                return false;

            if (m_CurrentEncodingMode == null)
                return false;



            IBMDStreamingMutableVideoEncodingMode em;

            string encodingModeName;
            m_CurrentEncodingMode.GetName(out encodingModeName);
            if (encodingModeName.StartsWith("Custom"))
                m_CurrentEncodingMode.CreateMutableVideoEncodingMode(out em);
            else
                em = (IBMDStreamingMutableVideoEncodingMode)m_CurrentEncodingMode;

            em.SetInt(_BMDStreamingEncodingModePropertyID.bmdStreamingEncodingPropertyVideoBitRateKbps, m_bitrate);

            m_streamingDeviceInput.SetVideoEncodingMode(em);

            em = null;

            m_streamingDeviceInput.GetVideoEncodingMode(out m_CurrentEncodingMode);
            m_CurrentEncodingMode.GetInt(_BMDStreamingEncodingModePropertyID.bmdStreamingEncodingPropertyVideoBitRateKbps, out m_bitrate);

            m_streamingDeviceInput.StartCapture();
            return true;
        }



        void IBMDStreamingDeviceNotificationCallback.StreamingDeviceArrived(IDeckLink device)
        {
            Debug.WriteLine("StreamingDeviceArrived");
            if (m_streamingDevice != null)
            {
                if (m_streamingDeviceInput != null)
                    m_streamingDeviceInput.SetCallback(null);
            }

            m_streamingDevice = device;
            m_streamingDeviceInput = (IBMDStreamingDeviceInput)m_streamingDevice;

            device.GetModelName(out m_deviceModelName);
            m_deviceMode = _BMDStreamingDeviceMode.bmdStreamingDeviceUnknown;

            m_streamingDeviceInput.SetCallback(this);

            UpdateForNewDevice();

            if (m_deviceMode != _BMDStreamingDeviceMode.bmdStreamingDeviceUnknown)
                UpdateForModeChange();

            return;
        }

        private void UpdateForModeChange()
        {
            Debug.WriteLine("UpdateForModeChange");
            switch (m_deviceMode)
            {
                case _BMDStreamingDeviceMode.bmdStreamingDeviceIdle:
                    m_status = "Idle";
                    break;
                case _BMDStreamingDeviceMode.bmdStreamingDeviceEncoding:
                    m_status = "Encoding";
                    break;
                case _BMDStreamingDeviceMode.bmdStreamingDeviceStopping:
                    m_status = "Stopping";
                    break;
            }

            m_DispalyNameWithStatus = "Device: " + m_deviceModelName + " " + m_status;

            Debug.WriteLine(m_DispalyNameWithStatus);

        }

        public void StopPreview()
        {
            Debug.WriteLine("StopPreview");
            m_playing = false;            
            if (m_streamingDeviceInput != null)
            {
                m_streamingDeviceInput.StopCapture();
            }

            if (m_sw != null)
            {
                m_sw.Close();
                m_sw = null;
            }


        }

        private void UpdateForNewDevice()
        {
            Debug.WriteLine("UpdateForNewDevice");
            IDeckLinkDisplayModeIterator inputModeIterator;
            m_CurrentEncodingMode = null;

            m_streamingDeviceInput.GetVideoInputModeIterator(out inputModeIterator);

            _BMDDisplayMode currentInputeModeValue;
            m_streamingDeviceInput.GetCurrentDetectedVideoInputMode(out currentInputeModeValue);

            IDeckLinkDisplayMode inputMode;
            inputModeIterator.Next(out inputMode);

            while (inputMode != null)
            {
                string modeName = string.Empty;
                if (inputMode.GetDisplayMode() == currentInputeModeValue)
                {

                    inputMode.GetName(out modeName);
                    m_displayModeName = modeName;
                    break;
                }
                inputModeIterator.Next(out inputMode);
            };

            inputMode = null;
            UpdateEncodingPresetsForInputMode();
            inputModeIterator = null;
        }







        private void UpdateEncodingPresetsForInputMode()
        {
            Debug.WriteLine("UpdateEncodingPresetsForInputMode");
            if (m_streamingDevice == null)
                return;

            _BMDDisplayMode inputMode = m_inputMode;
            m_encodingModes = new List<IBMDStreamingVideoEncodingMode>();

            IBMDStreamingVideoEncodingModePresetIterator presetIterator;
            m_streamingDeviceInput.GetVideoEncodingModePresetIterator(inputMode, out presetIterator);


            IBMDStreamingVideoEncodingMode encodingMode = null;
            presetIterator.Next(out encodingMode);
            while (encodingMode != null)
            {
                m_encodingModes.Add(encodingMode);
                presetIterator.Next(out encodingMode);
            }
            presetIterator = null;

            m_streamingDeviceInput.GetVideoEncodingMode(out m_CurrentEncodingMode);
            presetIterator.Next(out encodingMode);
        }

        void IBMDStreamingDeviceNotificationCallback.StreamingDeviceRemoved(IDeckLink device)
        {
            Debug.WriteLine("StreamingDeviceRemoved");
            if (device == m_streamingDevice)
            {
                //not sure if we need to nuke the callback
                m_streamingDeviceInput.SetCallback(null);

                m_streamingDeviceInput = null;
                m_streamingDevice = null;
                StopPreview();

                UpdateForNoDevice();
            }

            return;
        }

        void IBMDStreamingDeviceNotificationCallback.StreamingDeviceModeChanged(IDeckLink device, _BMDStreamingDeviceMode mode)
        {
            Debug.WriteLine("StreamingDeviceModeChanged");
            if (device != m_streamingDevice)
                return;

            if (mode == m_deviceMode)
                return;

            m_deviceMode = mode;

            UpdateForModeChange();

            return;
        }

        void IBMDStreamingH264InputCallback.H264NALPacketArrived(IBMDStreamingH264NALPacket nalPacket)
        {
            return;
        }

        void IBMDStreamingH264InputCallback.H264AudioPacketArrived(IBMDStreamingAudioPacket audioPacket)
        {

            return;
        }

        void IBMDStreamingH264InputCallback.MPEG2TSPacketArrived(IBMDStreamingMPEG2TSPacket tsPacket)
        {

            IntPtr by1 = IntPtr.Zero;

            try
            {
                tsPacket.GetBytes(out by1);



                int size = tsPacket.GetPayloadSize();

                if (size <= 0)
                    return;

                Byte[] byteArray1 = new Byte[size];
                System.Runtime.InteropServices.Marshal.Copy(by1, byteArray1, 0, size);

                if (m_sw != null)
                {

                    m_sw.Write(byteArray1, 0, size);
                }

            }
            catch (Exception e)
            {
                //logger.Error(e);
            }
            finally
            {

                System.Runtime.InteropServices.Marshal.ReleaseComObject(tsPacket);

            }
            return;
        }


        void IBMDStreamingH264InputCallback.H264VideoInputConnectorScanningChanged()
        {
            Debug.WriteLine("H264VideoInputConnectorScanningChanged");
            return;
        }

        void IBMDStreamingH264InputCallback.H264VideoInputConnectorChanged()
        {
            Debug.WriteLine("");
            return;
        }

        void IBMDStreamingH264InputCallback.H264VideoInputModeChanged()
        {
            Debug.WriteLine("H264VideoInputModeChanged");
            m_streamingDeviceInput.GetCurrentDetectedVideoInputMode(out m_inputMode);
            UpdateEncodingPresetsForInputMode();


            if (m_inputMode == _BMDDisplayMode.bmdModeUnknown)
            {
            }

            UpdateForModeChange();
            return;

        }
    }
}

